/* $Workfile: DspIoctl.h $   $Revision: 49 $    $Modtime: 2/07/05 2:49p $  */
#ifndef DSP_IOCTL_H
#define DSP_IOCTL_H
/*------------------------------------------------------- File Description -*\

  @doc PublicDspSdkApi PublicDsp55SdkApi PublicHwApi

  Odin TeleSystems Inc.

  SAA-1007-1 OTX DSP C54x Software Development Kit (SDK)
  SAA-1012-1 OTX DSP C55x Software Development Kit (SDK)

  @module

    DspIoctl.h | Macros and structs for sharing data between the DSP and the host. 
    (for all C5000 DSPs)

  @contents2
  DspIoctl.h |

  @index enum struct type | DspIoctl

  @contents2
  Communication protocol between Host and Dsp (DspIoctl.h) |

  @head3 DSP SDK Host .. DSP Macro, Constant and Type Reference |

  @ex IO Control Code structure |

     The IO Control codes used (from Host towards the DSP code) are
     16 bit values layed out as follows:


      1 1 1 1 1 1
      5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
     +-------------------------------+
     |S|C|P|         Code            |
     +-------------------------------+

     @ex where |

     S - SDK System IO Codes - reserved

         1 - SDK Reserved System Codes

         0 - User application code

     C - Data Complete - reserved

         0 - There is more data pending for this IO Code

         1 - Data is complete for this IO Code.

     P - Padded octet for Data

         0 - Even number of octets were transferred. Data length (in words)
             reflects actual number of data octets transferred (2*octets).

         1 - Odd number of octets were transferred. Data length includes one
             padded octet.

     Code - is the IO Control code value


     @ex Host-To-Dsp IO Control Communication |

     @ex There is only one pending IO Control packet in the DSP HPI memory
     at any time. The Otx Driver buffers pending IO Control packets. The DSP
     application must act on the IO Control packet with minimum latency. |

     @ex The DSP application processes the IO Control packets by performing
     these steps: |

     1. Retrieve (and possibly process) the data passed in the IO packet.
     2. Writing return data (if any) to OTXDSP_SDK_IOCTL_DATA_ADDR and data
        length OTXDSP_SDK_IOCTL_DATA_LEN_ADDR.
     3. Mark the completion of the packet by writing OTXDSP_SDK_IO_NO_COMMAND 
        to the OTXDSP_SDK_IOCTL_CODE_ADDR.
     4. Issue an interrupt towards the host. 

     @ex The data used for the IO Control communication is packed in the 
     following format. |


      1 1 1 1 1 1
      5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
     +-------------------------------+
     |           IO Code             |  OTXDSP_SDK_IOCTL_CODE_ADDR
     +-------------------------------+
     |  LogDevType   |  Handle       |  OTXDSP_SDK_IOCTL_HANDLE_ADDR
     +-------------------------------+
     |           Task Ref            |  OTXDSP_SDK_IOCTL_TASKREF_ADDR
     +-------------------------------+
     |           Data Length (n)     |  OTXDSP_SDK_IOCTL_DATA_LEN_ADDR
     +-------------------------------+
     |           Data Word 0         |  OTXDSP_SDK_IOCTL_DATA_ADDR
     +-------------------------------+
     |           Data Word 1         |
     +-------------------------------+
     :                               :
     +-------------------------------+
     |           Data Word (n-1)     |
     +-------------------------------+



     @ex Dsp-To-Host Asynchronous Event Communication |

     @ex The DSP code can buffer several Event packets in a circular buffer |

     @ex The data used for the Event communication from the DSP towards
     the host is packed in the following format. |


      1 1 1 1 1 1
      5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
     +-------------------------------+
     |           Processing Addr     |       OTXDSP_SDK_EVENT_BUF_PROCESSING_ADDR
     +-------------------------------+
     |S|C|P|     Event Code          |  >-+  OTXDSP_SDK_EVENT_BUF_ADDR (start of buffer)
     +-------------------------------+    |
     |  LogDevType   |  Handle       |    |
     +-------------------------------+    |
     |           TaskRef             |    |
     +-------------------------------+    |
     |           TimeStamp bit 48-63 |    |
     +-------------------------------+    |
     |           TimeStamp bit 32-47 |    |
     +-------------------------------+    |
     |           TimeStamp bit 16-31 |    |
     +-------------------------------+    |
     |           TimeStamp bit  0-15 |    |
     +-------------------------------+    |
     |           Data Length (n=0)   |    |
     +-------------------------------+    |
     |S|C|P|     Event Code          |    |
     +-------------------------------+    |
     |  LogDevType   |  Handle       |    |
     +-------------------------------+    |
     |           TaskRef             |    |
     +-------------------------------+    |
     |           TimeStamp bit 48-63 |    |
     +-------------------------------+    |
     |           TimeStamp bit 32-47 |    |
     +-------------------------------+    |
     |           TimeStamp bit 16-31 |    |
     +-------------------------------+    |
     |           TimeStamp bit  0-15 |    |
     +-------------------------------+    |
     |           Data Length (n=2)   |    |
     +-------------------------------+    |
     |           Data Word 0         |    |
     +-------------------------------+    |
     |           Data Word 1         |    |
     +-------------------------------+    |
     |  OTXDSP_SDK_EVENT_NO_COMMAND  |    |
     +-------------------------------+    |
     :                               :    |
     +-------------------------------+    |
     |                               |  <-+  End of Event buffer
     +-------------------------------+

   */ /*

  @contents2
  Constants, Macros, and Types (DspIoctl.h) |

  @head3 DSP C54x SDK Macro, Constant, and Type Reference |

  @index enum struct type | DspIoctl


\*--------------------------------------------------------------------------*/

 
#ifdef HDR_DBG // Define to trace include order of headers
#pragma message("\n" __FILE__ "(1): Including\n")
#endif

#if defined(__linux__) 
#pragma pack(2)
#elif defined(_TMS320C5XX)
// No pack pragma needed for the DSP compiler
#elif defined(__TMS320C55X__)
// No pack pragma needed for the DSP compiler
#elif defined(__BORLANDC__)
#pragma option push -a2
#else
#pragma pack(push,2)
#endif


// @type OTXDSP_LDEVICE_USER_APPLICATION | Reserved. The logical device type is the 8 most significant bits of the <t OTXDSP_SDK_IOCTL_HANDLE_ADDR> address
#define OTXDSP_SDK_LDEVICE_USER_APPLICATION     0x00

#define OTXDSP_SDK_MASK_LDEVICE_TYPE            0xFF00
#define OTXDSP_SDK_MASK_HANDLE                  0x00FF

#define OTXDSP_SDK_NO_LDEVICE                   0xFF00
#define OTXDSP_SDK_NO_HANDLE                    0x00FF

#define OTXDSP_SDK_MASK_EVENT_DATA_PADDED       0x2000
#define OTXDSP_SDK_MASK_EVENT_CODE              0x9FFF

#define OTXDSP_SDK_MASK_IO_DATA_PADDED          0x2000
#define OTXDSP_SDK_MASK_IO_DATA_COMPLETE        0x4000
#define OTXDSP_SDK_MASK_IO_CODE                 0x9FFF


// Reserved IO Control Codes
// @type OTXDSP_SDK_IO_NO_COMMAND | Reserved. Not to be used an IO Control Code. This value should be written to <t OTXDSP_SDK_IOCTL_CODE_ADDR> when the DSP completes the IO packet.
#define OTXDSP_SDK_IO_NO_COMMAND                0x8000

// Reserved IO Control Codes
// @type OTXDSP_SDK_IO_CREATE | Issued when OtxDrvCreateLogicalDevice() is called. Data Length 0
#define OTXDSP_SDK_IO_CREATE                    0x8001
// @type OTXDSP_SDK_IO_ENABLE | Issued when OtxDrvEnable() is called. Data Length 0
#define OTXDSP_SDK_IO_ENABLE                    0x8002
// @type OTXDSP_SDK_IO_DISABLE | Issued when OtxDrvDisable() is called. Data Length 0
#define OTXDSP_SDK_IO_DISABLE                   0x8003
// @type OTXDSP_SDK_IO_CONNECT | Issued when OtxDrvConnectLogicalDevice() is called. Data is <t OtxDspSdkIoInConnectS>.
#define OTXDSP_SDK_IO_CONNECT                   0x8004
// @type OTXDSP_SDK_IO_DISCONNECT | Issued when OtxDrvDisconnectLogicalDevice() is called. Data is <t OtxDspSdkIoInDisconnectS>.
#define OTXDSP_SDK_IO_DISCONNECT                0x8005
// @type OTXDSP_SDK_IO_RESET | Issued when <f OtxDrvReset> is called. Data Length 0
#define OTXDSP_SDK_IO_RESET                     0x8006
// @type OTXDSP_SDK_IO_INIT_BSP | Issued internally from the driver. DSP application should call <f OtxDspInitBufferedSerialPorts> when the driver sends this IO command. Data Length 0
#define OTXDSP_SDK_IO_INIT_BSP                  0x8007
// @type OTXDSP_SDK_IO_START_PROGRAM | Issued internally from the driver after the program is downloaded to the DSP. DSP application should call <f OtxDspStartProgram> when the driver sends this IO command. Data is <t OtxDspSdkIoInStartProgramS>.
#define OTXDSP_SDK_IO_START_PROGRAM             0x8008
// @type OTXDSP_SDK_IO_INIT_TIMER | Issued internally from the driver to synchronize the timers used for timestamping. Data Length is 0.
#define OTXDSP_SDK_IO_INIT_TIMER                0x8009
// @type OTXDSP_SDK_IO_CLOSE | Issued when OtxDrvClose() is called. Data Length 0
#define OTXDSP_SDK_IO_CLOSE                     0x800A
// @type OTXDSP_SDK_IO_GET_FRAME_BUFFER_COUNTER | Returns the current Frame Buffer Counter from the DSP. The size of the frame buffer is set by each DSP program.
#define OTXDSP_SDK_IO_GET_FRAME_BUFFER_COUNTER  0x800B



// Reserved Event Codes
// @type OTXDSP_SDK_EVENT_NO_COMMAND | Marks the end of the pending Events to be processed by the host
#define OTXDSP_SDK_EVENT_NO_COMMAND             0x8000


// @type OTXDSP_SDK_RX_DIRECTION | Otx DSP SDK Receive Direction, i.e. data for the DSP code to process
#define OTXDSP_SDK_RX_DIRECTION 0
// @type OTXDSP_SDK_TX_DIRECTION | Otx DSP SDK Transmit Direction, i.e. data generated by the DSP code
#define OTXDSP_SDK_TX_DIRECTION 1

// @type OTXDSP_SDK_SUCCESS | Generic test for success on any status value (non-negative numbers indicate success).
#define OTXDSP_SDK_SUCCESS(status) ((short)(status) >= 0)

// @type OTXDSP_SDK_FAILURE | Generic test for success on any status value (negative numbers indicate failure).
#define OTXDSP_SDK_FAILURE(status) ((short)(status) < 0)

/*--------------------------------------------------------------------------*\
  OTX DSP SDK General result codes
\*--------------------------------------------------------------------------*/

/*-------------------------------*\
  Success codes
\*-------------------------------*/
// @type OTXDSP_SDK_S_OK | Otx DSP SDK Result Code: Everything ok
#define OTXDSP_SDK_S_OK                        (0x0000)

// @type OTXDSP_SDK_S_FALSE | Otx DSP SDK Result Code: Everything ok, but result is logically false
#define OTXDSP_SDK_S_FALSE                     (0x0001)

// @type OTXDSP_SDK_S_SIGNALLED | Otx DSP SDK Result Code: An event was signalled
#define OTXDSP_SDK_S_SIGNALLED                 (0x0002)

// @type OTXDSP_SDK_S_TIMEOUT | Otx DSP SDK Result Code: An wait operation timed out
#define OTXDSP_SDK_S_TIMEOUT                   (0x0003)

// @type OTXDSP_SDK_S_PENDING | Otx DSP SDK Result Code: The data necessary to complete this operation is not yet available.
#define OTXDSP_SDK_S_PENDING                   (0x0004)

/*-------------------------------*\
  Failure codes
\*-------------------------------*/

// @type OTXDSP_SDK_E_UNEXPECTED | Otx DSP SDK Result Code: Catastrophic failure
#define OTXDSP_SDK_E_UNEXPECTED                (0xFFFF)

// @type OTXDSP_SDK_E_NOTIMPL | Otx DSP SDK Result Code: Not implemented
#define OTXDSP_SDK_E_NOTIMPL                   (0x8001)

// @type OTXDSP_SDK_E_OUTOFMEMORY | Otx DSP SDK Result Code: Creation (new/malloc) failed, i.e. ran out of memory
#define OTXDSP_SDK_E_OUTOFMEMORY               (0x8002)

// @type OTXDSP_SDK_E_OUT_OF_RESOURCES | Otx DSP SDK Result Code: Ran out of software resources
#define OTXDSP_SDK_E_OUT_OF_RESOURCES          (0x8003)

// @type OTXDSP_SDK_E_INVALIDARG | Otx DSP SDK Result Code: One or more arguments are invalid
#define OTXDSP_SDK_E_INVALIDARG                (0x8004)

// @type OTXDSP_SDK_E_POINTER | Otx DSP SDK Result Code: Invalid pointer
#define OTXDSP_SDK_E_POINTER                   (0x8005)

// @type OTXDSP_SDK_E_HANDLE | Otx DSP SDK Result Code: Invalid handle
#define OTXDSP_SDK_E_HANDLE                    (0x8006)

// @type OTXDSP_SDK_E_ABORT | Otx DSP SDK Result Code: Operation aborted
#define OTXDSP_SDK_E_ABORT                     (0x8007)

// @type OTXDSP_SDK_E_FAIL | Otx DSP SDK Result Code: Unspecified error
#define OTXDSP_SDK_E_FAIL                      (0x8008)

// @type OTXDSP_SDK_E_ACCESSDENIED | Otx DSP SDK Result Code: General access denied error
#define OTXDSP_SDK_E_ACCESSDENIED              (0x8009)

// @type OTXDSP_SDK_E_CALL_FAIL | Otx DSP SDK Result Code: Call into other process failed (e.g. app to driver)
#define OTXDSP_SDK_E_CALL_FAIL                 (0x800A)

// @type OTXDSP_SDK_E_NO_SUCH_OBJECT | Otx result Code: Reference to non-existent object or type, etc.
#define OTXDSP_SDK_E_NO_SUCH_OBJECT            (0x800B)

// @type OTXDSP_SDK_E_NOT_SUPPORTED | Otx DSP SDK Result Code: Functionality not supported by the logical device
#define OTXDSP_SDK_E_NOT_SUPPORTED             (0x800C)

// @type OTXDSP_SDK_E_REG_NO | Otx DSP SDK Result Code: Invalid Register Number
#define OTXDSP_SDK_E_REG_NO                    (0x800D)

// @type OTXDSP_SDK_E_TIMEOUT | Otx DSP SDK Result Code: Timeout
#define OTXDSP_SDK_E_TIMEOUT                   (0x800E)

// @type OTXDSP_SDK_E_BLOCKED | Otx DSP SDK Result Code: Device Blocked
#define OTXDSP_SDK_E_BLOCKED                   (0x800F)

// @type OTXDSP_SDK_E_NO_INTERRUPT | Otx DSP SDK Result Code: No Interrupt
#define OTXDSP_SDK_E_NO_INTERRUPT              (0x8010)

// @type OTXDSP_SDK_E_WRONG_CONTEXT | Otx DSP SDK Result Code: Call not allowed here
#define OTXDSP_SDK_E_WRONG_CONTEXT             (0x8011)

// @type OTXDSP_SDK_E_NOT_INITIALIZED | Otx DSP SDK Result Code: The OtxXxxxInit() function needs to be called for this logical device
#define OTXDSP_SDK_E_NOT_INITIALIZED           (0x8012)

// @type OTXDSP_SDK_E_ALREADY_EXISTS | Otx DSP SDK Result Code: Attempted to create an already existing Logical device
#define OTXDSP_SDK_E_ALREADY_EXISTS            (0x8013)

// @type OTXDSP_SDK_E_IN_USE | Otx DSP SDK Result Code: The requested resource is in use by another application
#define OTXDSP_SDK_E_IN_USE                    (0x8014)

// @type OTXDSP_SDK_E_IN_USE | Otx DSP SDK Result Code: The requested resource or function cannot service the request at this time
#define OTXDSP_SDK_E_BUSY                      (0x8015)

// @type OTXDSP_SDK_E_NOT_WORKING | Otx DSP SDK Result Code: The logical device is not in working state
#define OTXDSP_SDK_E_NOT_WORKING               (0x8016)

// @type OTXDSP_SDK_E_NOT_CONNECTED | Otx DSP SDK Result Code: The logical device is not connected
#define OTXDSP_SDK_E_NOT_CONNECTED             (0x8017)

// @type OTXDSP_SDK_E_OUT_OF_CONNECTIONS | Otx DSP SDK Result Code: Connection limit is reached. The DSP code cannot accept more connections until a connection is freed.
#define OTXDSP_SDK_E_OUT_OF_CONNECTIONS        (0x8018)

// @type OTXDSP_SDK_E_UNSUPPORTED_DEVICE | Otx DSP SDK Result Code: The DSP code cannot support the requested Logical Device
#define OTXDSP_SDK_E_UNSUPPORTED_DEVICE        (0x8019)

// @type OTXDSP_SDK_E_UNSUPPORTED_PIPE_CAPACITY | Otx DSP SDK Result Code: The DSP code cannot support the requested pipe capacity
#define OTXDSP_SDK_E_UNSUPPORTED_PIPE_CAPACITY (0x801A)

// @type OTXDSP_SDK_E_UNSUPPORTED_IO_CODE | Otx DSP SDK Result Code: The DSP code does not support this IO code
#define OTXDSP_SDK_E_UNSUPPORTED_IO_CODE       (0x801B)

// @type OTXDSP_SDK_E_UNSUPPORTED_VERSION | Otx DSP SDK Result Code: The DSP code does not support this driver version
#define OTXDSP_SDK_E_UNSUPPORTED_VERSION       (0x801C)




// @type OTXDSP_SDK_MAX_TIMESLOTS | Maximum number of timeslots on any OTX highway. Some DSP code will only support maximum 64 timeslots, and some DSP code will only support a maximum of 32 timeslots.
#define OTXDSP_SDK_MAX_TIMESLOTS 128  




/*******************************************************************************
  @devnote
   Structs for passing data over the DSP IOCTL interface
*******************************************************************************/

// @struct Standard return code struct
struct OtxDspSdkIoOutResultS
{
  unsigned short   nResult; // @field DSP Application return code
};
typedef struct OtxDspSdkIoOutResultS OtxDspSdkIoOutResultS;

// @struct Data for the <t OTXDSP_SDK_IO_CONNECT> IO Control code
struct OtxDspSdkIoInConnectS
{
  short        nDirection; // @field Either <t OTXDSP_SDK_RX_DIRECTION> or <t OTXDSP_SDK_TX_DIRECTION>
  short        nCapacity;  // @field Pipe capacity in kbytes/s
  unsigned short sqConnectMask[OTXDSP_SDK_MAX_TIMESLOTS/2];  // @field Timeslots to connect to on the DSP Highway
  unsigned short coMaskSize; // @field Actual number of timeslots used in sqConnectMask
  short        iConnectPoint; // @field is the <p iFromConnectPoint> or <p iToConnectPoint> parameter (the connection point referring to the logical device) from the <f OtxDrvConnectLogicalDevice> call

  unsigned short hPipeHandleLowPart;  // @field Lower 16 bits of the host's handle for this pipe
  unsigned short hPipeHandleHighPart; // @field Upper 16 bits of the host's handle for this pipe
};
typedef struct OtxDspSdkIoInConnectS OtxDspSdkIoInConnectS;

// @struct Data for the <t OTXDSP_SDK_IO_DISCONNECT> IO Control code
struct OtxDspSdkIoInDisconnectS
{
  short        nDirection; // @field Either <t OTXDSP_SDK_RX_DIRECTION> or <t OTXDSP_SDK_TX_DIRECTION>
  short        nCapacity;  // @field Pipe capacity in kbytes/s
  unsigned short sqConnectMask[OTXDSP_SDK_MAX_TIMESLOTS/2];  // @field Timeslots to connect to on the DSP Highway
  unsigned short coMaskSize; // @field Actual number of timeslots used in sqConnectMask
  short        iConnectPoint; // @field is the <p iFromConnectPoint> or <p iToConnectPoint> parameter (the connection point referring to the logical device) from the <f OtxDrvConnectLogicalDevice> call

  unsigned short hPipeHandleLowPart;  // @field Lower 16 bits of the host's handle for this pipe
  unsigned short hPipeHandleHighPart; // @field Upper 16 bits of the host's handle for this pipe
};
typedef struct OtxDspSdkIoInDisconnectS OtxDspSdkIoInDisconnectS;

// @struct Data for the <t OTXDSP_SDK_IO_START_PROGRAM> IO Control code
struct OtxDspSdkIoInStartProgramS
{
  unsigned short nClkMdVal;         // @field The PLLMUL, PLLDIV, and PLLCOUNT bits of the Clock Mode Register
  unsigned short nHighwaySpeed;     // @field DSP Serial port highway speed in kbps (2048, 4096, or 8192 kpbs)

  unsigned short nDrvRevNoMajor;    // @field Driver Revision; Major
  unsigned short nDrvRevNoMinor;    // @field Driver Revision; Minor
  unsigned short nDrvRevNoPr;       // @field Driver Revision; PR (PR = 0 = Preliminary, PR = 1 = Release, If PR = 1 P-Version is always 0)
  unsigned short nDrvRevNoPversion; // @field Driver Revision; P-Version: (Only valid if PR = 0)

  unsigned short nReserved0;        // @field Reserved for future use
  unsigned short nReserved1;        // @field Reserved for future use
  unsigned short nReserved2;        // @field Reserved for future use
  unsigned short nReserved3;        // @field Reserved for future use
  unsigned short nReserved4;        // @field Reserved for future use
  unsigned short nReserved5;        // @field Reserved for future use
  unsigned short nReserved6;        // @field Reserved for future use
  unsigned short nReserved7;        // @field Reserved for future use

};
typedef struct OtxDspSdkIoInStartProgramS OtxDspSdkIoInStartProgramS;


struct OtxDspSdkIoOutStartProgramS
{
  unsigned short nResult;   // Must be the first field in this struct

  unsigned short nEventBufAddr;
  unsigned short nEventBufLen;
  unsigned short nIoctlCodeAddr;
  unsigned short nIoctlDataMaxSize;
  unsigned short nReserved0;
  unsigned short nReserved1;
  unsigned short nReserved2;
  unsigned short nReserved3;
  unsigned short nReserved4;
  unsigned short nReserved5;
  unsigned short nReserved6;
  unsigned short nReserved7;
  unsigned short nReserved8;
  unsigned short nReserved9;
  unsigned short nReserved10;
};
typedef struct OtxDspSdkIoOutStartProgramS OtxDspSdkIoOutStartProgramS;


typedef struct OtxDspSdkIoOutGetFrameBufferCounterS
{
  unsigned short        nResult;
  unsigned short        nFrameBufferCounter;
  unsigned short        nReserved0;
  unsigned short        nReserved1;
  unsigned short        nReserved2;
  unsigned short        nReserved3;
} OtxDspSdkIoOutGetFrameBufferCounter;




#if defined(__linux__) 
#pragma pack(2)
#elif defined(_TMS320C5XX)
// No pack pragma needed for the DSP compiler
#elif defined(__TMS320C55X__)
// No pack pragma needed for the DSP compiler
#elif defined(__BORLANDC__)
#pragma option pop
#else
#pragma pack(pop)
#endif


/******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1999-2003


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

******************************************************************************/
#endif /* DSP_IOCTL_H */
